/*
 * File: main.c
 * Date: 15.08.2011
 * Denis Zheleznjakov http://ziblog.ru
 */

#include "main.h"

enum rcc_cfgr_mco_select
{
	RCC_CFGR_MCOSEL_DISABLED = 0,
	RCC_CFGR_MCOSEL_SYSCLK = RCC_CFGR_MCOSEL_0,
	RCC_CFGR_MCOSEL_HSI = RCC_CFGR_MCOSEL_1,
	RCC_CFGR_MCOSEL_MSI = RCC_CFGR_MCOSEL_1 | RCC_CFGR_MCOSEL_0,
	RCC_CFGR_MCOSEL_HSE = RCC_CFGR_MCOSEL_2,
	RCC_CFGR_MCOSEL_PLL = RCC_CFGR_MCOSEL_2 | RCC_CFGR_MCOSEL_0,
	RCC_CFGR_MCOSEL_LSI = RCC_CFGR_MCOSEL_2 | RCC_CFGR_MCOSEL_1,
	RCC_CFGR_MCOSEL_LSE = RCC_CFGR_MCOSEL_2 | RCC_CFGR_MCOSEL_1
			| RCC_CFGR_MCOSEL_0
};

#define SEG_A		BIT(6)
#define SEG_B		BIT(2)
#define SEG_C		BIT(5)
#define SEG_D		BIT(4)
#define SEG_E		BIT(0)
#define SEG_F		BIT(7)
#define SEG_G		BIT(3)
#define SEG_H		BIT(15)
#define SEG_J		BIT(14)
#define SEG_K		BIT(10)
#define SEG_M		BIT(1)
#define SEG_N		BIT(12)
#define SEG_P		BIT(8)
#define SEG_Q		BIT(11)
#define SEG_DP		BIT(13)
#define SEG_COL		BIT(9)

#define SYMBOL_0 	(uint16_t)(SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F)
#define SYMBOL_1 	(uint16_t)(SEG_B | SEG_C)
#define SYMBOL_2 	(uint16_t)(SEG_A | SEG_B | SEG_G | SEG_M | SEG_E | SEG_D)
#define SYMBOL_3 	(uint16_t)(SEG_A | SEG_B | SEG_C | SEG_D | SEG_G | SEG_M)
#define SYMBOL_4 	(uint16_t)(SEG_F | SEG_G | SEG_M | SEG_B | SEG_C)
#define SYMBOL_5 	(uint16_t)(SEG_A | SEG_F | SEG_G | SEG_M | SEG_C | SEG_D)
#define SYMBOL_6 	(uint16_t)(SEG_A | SEG_F | SEG_G | SEG_M | SEG_C | SEG_D | SEG_E)
#define SYMBOL_7 	(uint16_t)(SEG_A | SEG_B | SEG_C)
#define SYMBOL_8 	(uint16_t)(SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G | SEG_M)
#define SYMBOL_9 	(uint16_t)(SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G | SEG_M)
#define SYMBOL_A 	(uint16_t)(SEG_A | SEG_B | SEG_E | SEG_F | SEG_C | SEG_G | SEG_M)
#define SYMBOL_B 	(uint16_t)(SEG_F | SEG_E | SEG_D | SEG_C | SEG_G | SEG_M)
#define SYMBOL_C 	(uint16_t)(SEG_A | SEG_F | SEG_E | SEG_D)
#define SYMBOL_D 	(uint16_t)(SEG_B | SEG_C | SEG_D | SEG_E | SEG_G | SEG_M)
#define SYMBOL_E 	(uint16_t)(SEG_A | SEG_F | SEG_E | SEG_D | SEG_G | SEG_M)
#define SYMBOL_F 	(uint16_t)(SEG_A | SEG_F | SEG_E | SEG_G | SEG_M)

static const uint16_t mcu_lcd_symbols_decoder[] =
{ SYMBOL_0, SYMBOL_1, SYMBOL_2, SYMBOL_3, SYMBOL_4, SYMBOL_5, SYMBOL_6,
		SYMBOL_7, SYMBOL_8, SYMBOL_9, SYMBOL_A, SYMBOL_B, SYMBOL_C, SYMBOL_D,
		SYMBOL_E, SYMBOL_F };

void mcu_lcd_print(uint8_t position, uint16_t symbols)
{
	uint8_t i;

	switch (position)
	{
		case (0):
		{
			for (i = 0; i < 8; i += 2)
			{
				LCD->RAM[i] &= ~(BIT(0) | BIT(1) | BIT(28) | BIT(29));
				//
				LCD->RAM[i] |= (symbols & 0x01 ? BIT(0) : 0)
						| (symbols & 0x02 ? BIT(1) : 0)
						| (symbols & 0x04 ? BIT(28) : 0)
						| (symbols & 0x08 ? BIT(29) : 0);
				symbols >>= 4;
			}
			break;
		}
		case (1):
		{
			for (i = 0; i < 8; i += 2)
			{
				LCD->RAM[i] &= ~(BIT(2) | BIT(7) | BIT(26) | BIT(27));
				//
				LCD->RAM[i] |= (symbols & 0x01 ? BIT(2) : 0)
						| (symbols & 0x02 ? BIT(7) : 0)
						| (symbols & 0x04 ? BIT(26) : 0)
						| (symbols & 0x08 ? BIT(27) : 0);
				symbols >>= 4;
			}
			break;
		}
		case (2):
		{
			for (i = 0; i < 8; i += 2)
			{
				LCD->RAM[i] &= ~(BIT(8) | BIT(9) | BIT(24) | BIT(25));
				//
				LCD->RAM[i] |= (symbols & 0x01 ? BIT(8) : 0)
						| (symbols & 0x02 ? BIT(9) : 0)
						| (symbols & 0x04 ? BIT(24) : 0)
						| (symbols & 0x08 ? BIT(25) : 0);
				symbols >>= 4;
			}
			break;
		}
		case (3):
		{
			for (i = 0; i < 8; i += 2)
			{
				LCD->RAM[i] &= ~(BIT(10) | BIT(11) | BIT(20) | BIT(21));
				//
				LCD->RAM[i] |= (symbols & 0x01 ? BIT(10) : 0)
						| (symbols & 0x02 ? BIT(11) : 0)
						| (symbols & 0x04 ? BIT(20) : 0)
						| (symbols & 0x08 ? BIT(21) : 0);
				symbols >>= 4;
			}
			break;
		}
		case (4):
		{
			for (i = 0; i < 8; i += 2)
			{
				LCD->RAM[i] &= ~(BIT(12) | BIT(13) | BIT(18) | BIT(19));
				//
				LCD->RAM[i] |= (symbols & 0x01 ? BIT(12) : 0)
						| (symbols & 0x02 ? BIT(13) : 0)
						| (symbols & 0x04 ? BIT(18) : 0)
						| (symbols & 0x08 ? BIT(19) : 0);
				symbols >>= 4;
			}
			break;
		}
		case (5):
		{
			for (i = 0; i < 8; i += 2)
			{
				LCD->RAM[i] &= ~(BIT(14) | BIT(15) | BIT(17) | BIT(16));
				//
				LCD->RAM[i] |= (symbols & 0x01 ? BIT(14) : 0)
						| (symbols & 0x02 ? BIT(15) : 0)
						| (symbols & 0x04 ? BIT(17) : 0)
						| (symbols & 0x08 ? BIT(16) : 0);
				symbols >>= 4;
			}
			break;
		}
	}
}

void mcu_lcd_print_hex(uint16_t data)
{
	// ожидаем завершения предыдущего вывода данных
	while (LCD->SR & LCD_SR_UDR)
	{
	}

	mcu_lcd_print(3, mcu_lcd_symbols_decoder[data & 0x0F]);
	mcu_lcd_print(2, mcu_lcd_symbols_decoder[(data >> 4) & 0x0F]);
	mcu_lcd_print(1, mcu_lcd_symbols_decoder[(data >> 8) & 0x0F]);
	mcu_lcd_print(0, mcu_lcd_symbols_decoder[(data >> 12) & 0x0F]);

	// запрос вывода новых данных
	LCD->SR |= LCD_SR_UDR;
}

//------------------------------------------------------------------------------
int main(void)
{
	uint8_t eeprom_data;

	RCC->AHBENR |= RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN;
	RCC->APB1ENR |= RCC_APB1ENR_LCDEN | RCC_APB1ENR_PWREN | RCC_APB1ENR_I2C1EN;

	// включаем LSI-генератор
	RCC->CSR |= RCC_CSR_LSION;
	while (!(RCC->CSR & RCC_CSR_LSIRDY))
	{
	}

	// светодиоды
	//PIN_CONFIGURATION(LED_GREEN);
	//PIN_CONFIGURATION(LED_BLUE);

	// кнопка
	PIN_CONFIGURATION(USER_BUTTON);

	// ЖКИ индикатор
	PIN_CONFIGURATION(PIN_LCDSEG0);
	PIN_CONFIGURATION(PIN_LCDSEG1);
	PIN_CONFIGURATION(PIN_LCDSEG2);
	PIN_CONFIGURATION(PIN_LCDSEG3);
	PIN_CONFIGURATION(PIN_LCDSEG4);
	PIN_CONFIGURATION(PIN_LCDSEG5);
	PIN_CONFIGURATION(PIN_LCDSEG6);
	PIN_CONFIGURATION(PIN_LCDSEG7);
	PIN_CONFIGURATION(PIN_LCDSEG8);
	PIN_CONFIGURATION(PIN_LCDSEG9);
	PIN_CONFIGURATION(PIN_LCDSEG10);
	PIN_CONFIGURATION(PIN_LCDSEG11);
	PIN_CONFIGURATION(PIN_LCDSEG12);
	PIN_CONFIGURATION(PIN_LCDSEG13);
	PIN_CONFIGURATION(PIN_LCDSEG14);
	PIN_CONFIGURATION(PIN_LCDSEG15);
	PIN_CONFIGURATION(PIN_LCDSEG16);
	PIN_CONFIGURATION(PIN_LCDSEG17);
	PIN_CONFIGURATION(PIN_LCDSEG18);
	PIN_CONFIGURATION(PIN_LCDSEG19);
	PIN_CONFIGURATION(PIN_LCDSEG20);
	PIN_CONFIGURATION(PIN_LCDSEG21);
	PIN_CONFIGURATION(PIN_LCDSEG22);
	PIN_CONFIGURATION(PIN_LCDSEG23);
	PIN_CONFIGURATION(LCDCOM0);
	PIN_CONFIGURATION(LCDCOM1);
	PIN_CONFIGURATION(LCDCOM2);
	PIN_CONFIGURATION(LCDCOM3);

	//
	PIN_CONFIGURATION(PIN_I2C_SCL);
	PIN_CONFIGURATION(PIN_I2C_SDA);

	// снимаем блокировку
	PWR->CR |= PWR_CR_DBP;

	// выбираем LSI генератор в качестве источника тактовых сигналов для ЖКИ (RTC)
	RCC->CSR |= RCC_CSR_RTCSEL_LSI;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	LCD->CR &= ~LCD_CR_BIAS;
	LCD->CR = LCD_CR_BIAS_1;
	//
	LCD->CR &= ~LCD_CR_DUTY;
	LCD->CR |= LCD_CR_DUTY_1 | LCD_CR_DUTY_0;

	LCD->FCR &= ~LCD_FCR_PS;
	LCD->FCR |= (4 << 22);
	//
	LCD->FCR &= ~LCD_FCR_DIV;
	LCD->FCR |= (1 << 18);
	//
	LCD->FCR &= ~LCD_FCR_CC;
	LCD->FCR |= LCD_FCR_CC_1;//| LCD_FCR_CC_0 | LCD_FCR_CC_2;

	while ((LCD->SR & LCD_SR_FCRSR) == 0)
	{
	}

	//
	LCD->CR &= ~LCD_CR_VSEL;
	LCD->CR |= LCD_CR_MUX_SEG;

	//
	LCD->CR |= LCD_CR_LCDEN;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// текущее значение частоты PCLK1 (может быть от 2 до 32, включительно)
	I2C1->CR2 &= ~I2C_CR2_FREQ;
	I2C1->CR2 |= 2;

	// установка частоты обмена
	I2C1->CCR &= ~I2C_CCR_CCR;
	I2C1->CCR |= 10;

	// время установления сигнала
	I2C1->TRISE = 3;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// разрешаем работу
	I2C1->CR1 |= I2C_CR1_PE;

	// формирование сигнала "Старт"
	I2C1->CR1 |= I2C_CR1_START;

	// ждем окончания формирования сигнала "Старт"
	while (!(I2C1->SR1 & I2C_SR1_SB))
	{
	}
	(void) I2C1->SR1;

	// передаем адрес ведомого + запись
	I2C1->DR = 0xA0;

	// ожидаем окончания передачи адреса
	while (!(I2C1->SR1 & I2C_SR1_ADDR))
	{
	}
	(void) I2C1->SR1;
	(void) I2C1->SR2;

	// передаем адрес десятой ячейки
	I2C1->DR = 10;
	// ожидаем окончания передачи адреса
	while (!(I2C1->SR1 & I2C_SR1_BTF))
	{
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// формирование сигнала "Повторный Старт"
	I2C1->CR1 |= I2C_CR1_START;

	// ждем окончания формирования сигнала "Повторный Старт"
	while (!(I2C1->SR1 & I2C_SR1_SB))
	{
	}
	(void) I2C1->SR1;

	// передаем адрес ведомого + чтение
	I2C1->DR = 0xA1;

	// ожидаем окончания передачи адреса
	while (!(I2C1->SR1 & I2C_SR1_ADDR))
	{
	}
	(void) I2C1->SR1;
	(void) I2C1->SR2;

	// ожидаем окончания приема данных
	while (!(I2C1->SR1 & I2C_SR1_RXNE))
	{
	}

	// cчитываем приянтое значение
	eeprom_data = I2C1->DR;

	// формирование сигнала "Стоп"
	I2C1->CR1 |= I2C_CR1_STOP;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	mcu_lcd_print_hex(eeprom_data);

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// формирование сигнала "Старт"
	I2C1->CR1 |= I2C_CR1_START;

	// ждем окончания формирования сигнала "Старт"
	while (!(I2C1->SR1 & I2C_SR1_SB))
	{
	}
	(void) I2C1->SR1;

	// передаем адрес ведомого + запись
	I2C1->DR = 0xA0;

	// ожидаем окончания передачи адреса
	while (!(I2C1->SR1 & I2C_SR1_ADDR))
	{
	}
	(void) I2C1->SR1;
	(void) I2C1->SR2;

	// передаем адрес десятой ячейки
	I2C1->DR = 10;
	// ожидаем окончания передачи адреса
	while (!(I2C1->SR1 & I2C_SR1_BTF))
	{
	}

	// передаем данные
	I2C1->DR = ++eeprom_data;
	// ожидаем окончания передачи адреса
	while (!(I2C1->SR1 & I2C_SR1_BTF))
	{
	}

	// формирование сигнала "Стоп"
	I2C1->CR1 |= I2C_CR1_STOP;

	while (1)
	{
	}

	return 0;
}
